/*
 *      interface.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "main.h"
#include "listProducts.h"
#include "interface.h"
#include "callbacks.h"
#include "config.h"
#include "preferences.h"
#include "print.h"

void initWindow(glda *pGlda){
	GtkWidget *pVBox;
  GtkWidget *pHBox;
  GtkWidget *pMenuBar;
  GtkWidget *pCategory;
  GtkWidget *pAddDellPrint;
  GtkWidget *pListProducts;
  gchar *windowTitle = NULL;
  
  if(pGlda->isSave){
  	if(pGlda->filesave)
  		windowTitle = g_strdup_printf("%s - Glda", pGlda->filesave);
  	else
  		windowTitle = g_strdup_printf("newfile.lda - Glda");
  }else{
  	if(pGlda->filesave)
  		windowTitle = g_strdup_printf("*%s - Glda", pGlda->filesave);
  	else
  		windowTitle = g_strdup_printf("*newfile.lda - Glda");
  }
  
  /*** main window ***/
  pGlda->pWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position(GTK_WINDOW(pGlda->pWindow), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size(GTK_WINDOW(pGlda->pWindow), 900, 500);
  gtk_window_set_title(GTK_WINDOW(pGlda->pWindow), windowTitle);
  gtk_window_set_icon_from_file(GTK_WINDOW(pGlda->pWindow), ICON_GLDA, NULL);
  
  g_signal_connect(G_OBJECT(pGlda->pWindow), "destroy", G_CALLBACK(OnDestroy), (glda*)pGlda);
  g_signal_connect(G_OBJECT(pGlda->pWindow), "delete-event", G_CALLBACK(OnQuit), (glda*)pGlda);
  
  pVBox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(pGlda->pWindow), pVBox);
  
  /*** menu bar ***/
  pMenuBar = create_pMenuBar(pGlda);
  gtk_box_pack_start(GTK_BOX(pVBox), pMenuBar, FALSE, FALSE, 0);
  
  pHBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(pVBox), pHBox, TRUE, TRUE, 0);
  
  /*** category and subcategory ***/
  pCategory = create_pCategory(pGlda);
  gtk_box_pack_start(GTK_BOX(pHBox), pCategory, TRUE, TRUE, 0);
  
  /*** button add/dell/print ***/
  pAddDellPrint = create_pAddDellPrint(pGlda);
  gtk_box_pack_start(GTK_BOX(pHBox), pAddDellPrint, FALSE, FALSE, 10);
  
  /*** list of products ***/
  pListProducts = create_pListProductsAdded(pGlda);
  gtk_box_pack_start(GTK_BOX(pHBox), pListProducts, TRUE, TRUE, 0);
  
  /*** Status bar ***/
  pGlda->pStatusBar = gtk_statusbar_new();
  gtk_box_pack_start(GTK_BOX(pVBox), pGlda->pStatusBar, FALSE, FALSE, 0);
  gtk_statusbar_push(GTK_STATUSBAR(pGlda->pStatusBar), GPOINTER_TO_INT(0), pGlda->pTextStatusbar);
  
  g_free(windowTitle);
}

/*** menu bar ***/
GtkWidget* create_pMenuBar(glda *pGlda){
  GtkWidget *pMenuBar;
  GtkWidget *pMenu;
  GtkAccelGroup *pAccel;
  GtkWidget *pMenuItem;
  
  pMenuBar = gtk_menu_bar_new();
  
  /** File menu **/
  pMenu = gtk_menu_new();
  pAccel = gtk_accel_group_new();
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_NEW, pAccel);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnNew), (GtkWidget*) pGlda);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN, pAccel);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnOpen), (GtkWidget*) pGlda);
  
  pMenuItem = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE, pAccel);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnSave), (glda*)pGlda);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE_AS, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnSaveAs), (glda*)pGlda);
  
  pMenuItem = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_PRINT, pAccel);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnPrint), (glda*)pGlda);
  
  pMenuItem = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, pAccel);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnDestroy), (glda*)pGlda);
  
  pMenuItem = gtk_menu_item_new_with_mnemonic("_Fichier");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(pMenuItem), pMenu);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenuBar), pMenuItem);
  
  /* Association shortcuts with the window */
  gtk_window_add_accel_group(GTK_WINDOW(pGlda->pWindow), pAccel);
  
  /** Edit menu **/
  pMenu = gtk_menu_new();
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnAdd), (glda*) pGlda);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnRemove), (glda*) pGlda);
  
  pMenuItem = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnPreferences), (glda*) pGlda);
  
  pMenuItem = gtk_menu_item_new_with_mnemonic("É_dition");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(pMenuItem), pMenu);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenuBar), pMenuItem);
  
  /** help menu **/
  pMenu = gtk_menu_new();
  
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnAbout), (glda*) pGlda);
  
  pMenuItem = gtk_menu_item_new_with_label("?");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(pMenuItem), pMenu);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenuBar), pMenuItem);
  
  return pMenuBar;
}

/*** category and subcategory ***/
GtkWidget* create_pCategory(glda *pGlda){
	GtkWidget *hpaned;
  GtkWidget *pVBox;
  GtkWidget *pScrollbar;
  GtkWidget *pTreeView;
  GtkTreeViewColumn *pColumn;
  GtkCellRenderer *pCellRenderer;
  gchar *pText;
  
  hpaned = gtk_hpaned_new();
  
  /*** category and subcategory ***/
  pVBox = gtk_vbox_new(TRUE, 0);
  gtk_paned_pack1(GTK_PANED (hpaned), pVBox, TRUE, FALSE);
  pScrollbar = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(pVBox), pScrollbar, TRUE, TRUE, 0);
  
  pGlda->pTreeStoreCategory = gtk_tree_store_new(N_COLUMN_CATEGORY, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
  create_ListCategory(pGlda);
  
  
  pTreeView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pGlda->pTreeStoreCategory));
  /* allow to search by category's name */
  gtk_tree_view_set_search_column(GTK_TREE_VIEW(pTreeView), CATEGORY_COLUMN);
  /* single selection */
  pGlda->selectCategory = gtk_tree_view_get_selection(GTK_TREE_VIEW(pTreeView));
  gtk_tree_selection_set_mode(pGlda->selectCategory, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(pGlda->selectCategory), "changed", G_CALLBACK(OnSelectSubCategory), pGlda);
  
  /* category's columns */
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Categories", pCellRenderer, "text", CATEGORY_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id(pColumn, CATEGORY_COLUMN);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("RefCat", pCellRenderer, "text", REF_CAT_COLUMN, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("RefSubCat", pCellRenderer, "text", REF_SUBCAT_COLUMN, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  gtk_container_add(GTK_CONTAINER(pScrollbar), pTreeView);
  
  /*** products ***/
  pVBox = gtk_vbox_new(TRUE, 0);
  gtk_paned_pack2(GTK_PANED (hpaned), pVBox, TRUE, FALSE);
  pScrollbar = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(pVBox), pScrollbar, TRUE, TRUE, 0);
  
  pGlda->pTreeStoreProducts = gtk_list_store_new(N_COLUMN_PRODUCTS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
  pTreeView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pGlda->pTreeStoreProducts));
  g_signal_connect(G_OBJECT(pTreeView), "row-activated", G_CALLBACK(OnDoubleClickProduct), pGlda);
  /* allow to search by product's name */
  gtk_tree_view_set_search_column(GTK_TREE_VIEW(pTreeView), PRODUCTS_COLUMN);
  /* multiple selection */
  pGlda->selectProducts = gtk_tree_view_get_selection(GTK_TREE_VIEW(pTreeView));
  gtk_tree_selection_set_mode(pGlda->selectProducts, GTK_SELECTION_MULTIPLE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Articles", pCellRenderer, "text", PRODUCTS_COLUMN, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_min_width(pColumn, 100);
  gtk_tree_view_column_set_sort_column_id(pColumn, PRODUCTS_COLUMN);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  
  /* product's column */
  pText = g_strdup_printf("Prix (en %s)", g_key_file_get_string(pGlda->pConfig, "general", "devise", NULL));
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes(pText, pCellRenderer, "text", PRICE_PRODUCTS_COLUMN, NULL);
  pGlda->pColumnPriceProducts = pColumn;
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_min_width(pColumn, 50);
  gtk_tree_view_column_set_sort_column_id(pColumn, PRICE_PRODUCTS_COLUMN);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  g_free(pText);
  if(!g_key_file_get_boolean(pGlda->pConfig, "product", "show_column_price", NULL))
  	gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Ref", pCellRenderer, "text", REF_PRODUCTS_COLUMN, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  gtk_container_add(GTK_CONTAINER(pScrollbar), pTreeView);
  
  return hpaned;
}

void create_ListCategory(glda *pGlda){
	GtkTreeIter pIterCategory;
  GtkTreeIter pIterSubCategory;
  gchar *pCategory = NULL;
  gchar *pSubCategory = NULL;
  gchar *pRefCategory = NULL;
  gchar *pRefSubCategory = NULL;
  gint positionCategory, positionSubCategory;
  gint errorCategory, errorSubCategory;
  
  /* add category */
  printf("Show the list of category and subcategory\n");
  positionCategory = 1;
  do{
  	if(pCategory != NULL){
  		g_free(pCategory);
  		g_free(pRefCategory);
  		pCategory = NULL;
  		pRefCategory = NULL;
  	}
  	
  	errorCategory = getCategoryByPosition(pGlda->xmlDocListProducts, positionCategory, &pCategory, &pRefCategory);
  	if(errorCategory == CATEGORY_NO_ERROR){
  		gtk_tree_store_append(pGlda->pTreeStoreCategory, &pIterCategory, NULL);
  		gtk_tree_store_set(pGlda->pTreeStoreCategory, &pIterCategory,
  											CATEGORY_COLUMN, pCategory,
  											REF_CAT_COLUMN, pRefCategory,
  											REF_SUBCAT_COLUMN, "0",
  											-1);
  	}
  	
  	/* add subcategory */
  	if(errorCategory == CATEGORY_NO_ERROR){
  		positionSubCategory = 1;
	  	do{
  			if(pSubCategory != NULL){
  				g_free(pSubCategory);
  				g_free(pRefSubCategory);
  				pSubCategory = NULL;
					pRefSubCategory = NULL;
				}
				
  			errorSubCategory = getSubCategoryByPosition(pGlda->xmlDocListProducts,
  																									positionCategory, positionSubCategory,
  																									&pSubCategory, &pRefSubCategory);
  			if(errorSubCategory == CATEGORY_NO_ERROR){
  				gtk_tree_store_append(pGlda->pTreeStoreCategory, &pIterSubCategory, &pIterCategory);
  				gtk_tree_store_set(pGlda->pTreeStoreCategory, &pIterSubCategory,
  													CATEGORY_COLUMN, pSubCategory,
  													REF_CAT_COLUMN, pRefCategory,
  													REF_SUBCAT_COLUMN, pRefSubCategory,
  													-1);
  			}
  			
  			positionSubCategory++;
  		}while(errorSubCategory == CATEGORY_NO_ERROR);
  	}
  	
  	positionCategory++;
  }while(errorCategory == CATEGORY_NO_ERROR);
}

void create_pListProductsByReference(glda *pGlda, gchar *pRefCategory, gchar *pRefSubCategory){
	GtkTreeIter pIterProduct;
	gchar *pCategory = NULL;
  gchar *pRef = NULL;
  gchar *pProductName = NULL;
  gchar *pProductPrice = NULL;
  gchar *pRefProduct = NULL;
	gint positionCategory, positionSubCategory, positionProduct;
	gint error;
	
	/* Get position of category where the reference is "pRefCategory" */
	positionCategory = 1;
	do{
		if(pCategory != NULL){
  		g_free(pCategory);
  		g_free(pRef);
  		pCategory = NULL;
  		pRef = NULL;
  	}
  	error = getCategoryByPosition(pGlda->xmlDocListProducts, positionCategory, &pCategory, &pRef);
  	positionCategory++;
	}while(error == CATEGORY_NO_ERROR && strcmp(pRefCategory, pRef) != 0);
	positionCategory--;
	
	/* Get position of subcategory where the reference is "pRefSubCategory" */
	positionSubCategory = 1;
	do{
		if(pCategory != NULL){
			g_free(pCategory);
			g_free(pRef);
			pCategory = NULL;
			pRef = NULL;
		}
		error = getSubCategoryByPosition(pGlda->xmlDocListProducts,
																			positionCategory, positionSubCategory,
																			&pCategory, &pRef);
		positionSubCategory++;
	}while(error == CATEGORY_NO_ERROR && strcmp(pRefSubCategory, pRef) != 0);
	positionSubCategory--;
	
	/* clear the list of products before show the new */
	gtk_list_store_clear(pGlda->pTreeStoreProducts);
	
	printf("Show list of products (refcat=%s/refsubcat=%s).\n", pRefCategory, pRefSubCategory);
	positionProduct = 1;
	do{
		if(pProductName != NULL){
			g_free(pProductName);
			g_free(pProductPrice);
			g_free(pRefProduct);
			pProductName = NULL;
			pProductPrice = NULL;
			pRefProduct = NULL;
		}
		
		error = getProductReferenceByPosition(pGlda->xmlDocListProducts, positionCategory, positionSubCategory, positionProduct, &pRefProduct);
		getProductNameByPosition(pGlda->xmlDocListProducts, positionCategory, positionSubCategory, positionProduct, &pProductName);
		getProductPriceByPosition(pGlda->xmlDocListProducts, positionCategory, positionSubCategory, positionProduct, &pProductPrice);
		
		if(error == PRODUCT_NO_ERROR){
			gtk_list_store_append(pGlda->pTreeStoreProducts, &pIterProduct);
			gtk_list_store_set(pGlda->pTreeStoreProducts, &pIterProduct,
													PRODUCTS_COLUMN, pProductName,
													PRICE_PRODUCTS_COLUMN, pProductPrice,
													REF_PRODUCTS_COLUMN, pRefProduct,
													-1);
		}
		
		positionProduct++;
	}while(error == PRODUCT_NO_ERROR);
}

/*** buttons Add/Dell/Print ***/
GtkWidget* create_pAddDellPrint(glda *pGlda){
  GtkWidget *pVBox;
  GtkWidget *pButton;
  
  pVBox = gtk_vbox_new(TRUE, 0);
  pButton = gtk_button_new_from_stock(GTK_STOCK_ADD);
  gtk_box_pack_start(GTK_BOX(pVBox), pButton, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnAdd), pGlda);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
  gtk_box_pack_start(GTK_BOX(pVBox), pButton, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnRemove), pGlda);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_PRINT);
  gtk_box_pack_start(GTK_BOX(pVBox), pButton, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnPrint), pGlda);
  
  return pVBox;
}

/*** list of products added ***/
GtkWidget* create_pListProductsAdded(glda *pGlda){
	GtkWidget *pVBox;
  GtkWidget *pTable;
  GtkWidget *pScrollbar;
  GtkWidget *pListView;
  GtkTreeViewColumn *pColumn;
  GtkCellRenderer *pCellRenderer;
  GtkWidget *pLabel;
  GtkAdjustment *adj;
  gchar *pText;
  
  pVBox = gtk_vbox_new(FALSE, 0);
  
  pLabel = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(pLabel), "<b>Liste de course</b>");
  gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_RIGHT);
  gtk_box_pack_start(GTK_BOX(pVBox), pLabel, FALSE, TRUE, 5);
  
  /* scrollbar */
  pScrollbar = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(pVBox), pScrollbar, TRUE, TRUE, 0);
  
  pGlda->pTreeStoreProductsAdded = gtk_list_store_new(N_COLUMN_PRODUCTS_ADDED,
  																	G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
  																	G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
  
  pListView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pGlda->pTreeStoreProductsAdded));
  /* allow to search by product's name */
  gtk_tree_view_set_search_column(GTK_TREE_VIEW(pListView), PRODUCTS_ADDED_COLUMN);
  /* multiple selection */
  pGlda->selectProductsAdded = gtk_tree_view_get_selection(GTK_TREE_VIEW(pListView));
  gtk_tree_selection_set_mode(pGlda->selectProductsAdded, GTK_SELECTION_MULTIPLE);
  
  /* column */
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Articles", pCellRenderer, "text", PRODUCTS_ADDED_COLUMN, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_min_width(pColumn, 200);
  gtk_tree_view_column_set_sort_column_id(pColumn, PRODUCTS_ADDED_COLUMN);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
    
  /* allow to edit quantity's column */
  adj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 2.0, 2.0));
  pCellRenderer = gtk_cell_renderer_spin_new();
  g_object_set(pCellRenderer, "editable", TRUE, "adjustment", adj, "digits", 0, NULL);
  pColumn = gtk_tree_view_column_new_with_attributes("Quantité", pCellRenderer, "text", QUANTITY_PRODUCTS_ADDED_COLUMN, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_min_width(pColumn, 50);
  g_signal_connect(G_OBJECT(pCellRenderer), "edited", G_CALLBACK(OnEditQuantity), (glda*)pGlda);
  gtk_tree_view_column_set_sort_column_id(pColumn, QUANTITY_PRODUCTS_ADDED_COLUMN);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
  pGlda->pColumnQuantityProductsAdded = pColumn;
  if(!g_key_file_get_boolean(pGlda->pConfig, "product", "show_column_quantity", NULL))
  	gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pText = g_strdup_printf("Prix (en %s)", g_key_file_get_string(pGlda->pConfig, "general", "devise", NULL));
  pColumn = gtk_tree_view_column_new_with_attributes(pText, pCellRenderer, "text", PRICE_PRODUCTS_ADDED_COLUMN, NULL);
  pGlda->pColumnPriceProductsAdded = pColumn;
  g_free(pText);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_min_width(pColumn, 50);
  gtk_tree_view_column_set_sort_column_id(pColumn, PRICE_PRODUCTS_ADDED_COLUMN);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
	if(!g_key_file_get_boolean(pGlda->pConfig, "product", "show_column_price", NULL))
  	gtk_tree_view_column_set_visible(pColumn, FALSE);
  
	pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("RefCat", pCellRenderer, "text", REF_CAT_PRODUCTS_ADDED_COLUMN, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("RefSSCat", pCellRenderer, "text", REF_SUBCAT_PRODUCTS_ADDED_COLUMN, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("RefArt", pCellRenderer, "text", REF_PRODUCTS_ADDED_COLUMN, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  gtk_container_add(GTK_CONTAINER(pScrollbar), pListView);
  
  /* Total */
  pTable = gtk_table_new(2, 2, FALSE);
  
  gtk_box_pack_start(GTK_BOX(pVBox), pTable, FALSE, TRUE, 5);
  
  if(g_key_file_get_boolean(pGlda->pConfig, "product", "show_total_quantity", NULL)){
		pLabel = gtk_label_new(NULL);
  	gtk_label_set_markup(GTK_LABEL(pLabel), _("<b>Nombre d'article(s) : </b>"));
  	gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_RIGHT);
	  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
	  gtk_misc_set_alignment(GTK_MISC(pLabel), 0, 0.5);
  
	  pText = g_strdup_printf("%d", pGlda->nbTotalProducts);
	  pGlda->pLabelNumberTotalProducts = gtk_label_new(pText);
	  gtk_label_set_justify(GTK_LABEL(pGlda->pLabelNumberTotalProducts), GTK_JUSTIFY_RIGHT);
	  gtk_table_attach_defaults(GTK_TABLE(pTable), pGlda->pLabelNumberTotalProducts, 1, 2, 0, 1);
	  gtk_misc_set_alignment(GTK_MISC(pGlda->pLabelNumberTotalProducts), 0, 0.5);
	  g_free(pText);
  }
  
  if(g_key_file_get_boolean(pGlda->pConfig, "product", "show_total_price", NULL)){
	  pLabel = gtk_label_new(NULL);
  	gtk_label_set_markup(GTK_LABEL(pLabel), "<b>Total : </b>");
  	gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_RIGHT);
  	gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 1, 2);
  	gtk_misc_set_alignment(GTK_MISC(pLabel), 0, 0.5);
  	
  	pText = g_strdup_printf("%.2f %s", pGlda->priceTotalProducts, g_key_file_get_string(pGlda->pConfig, "general", "devise", NULL));
  	pGlda->pLabelPriceTotalProducts = gtk_label_new(pText);
  	gtk_label_set_justify(GTK_LABEL(pGlda->pLabelPriceTotalProducts), GTK_JUSTIFY_RIGHT);
  	gtk_table_attach_defaults(GTK_TABLE(pTable), pGlda->pLabelPriceTotalProducts, 1, 2, 1, 2);
  	gtk_misc_set_alignment(GTK_MISC(pGlda->pLabelPriceTotalProducts), 0, 0.5);
  	g_free(pText);
  }
  
  return pVBox;
}

void addProduct(glda *pGlda){
	GList *listSubCategorySelected;
  GList *listProductsSelected;
  gint NbProductsSelected;
  gint *tabIndices = NULL;
  GtkTreeIter Iter;
  GtkTreePath *pPath;
  gchar *pRefCategory = NULL;
  gchar *pRefSubCategory = NULL;
  gchar *pProductName = NULL;
  gchar *pProductPrice = NULL;
  gchar *pRefProduct = NULL;
  gchar *pRefCategorySearch = NULL;
  gchar *pRefSubCategorySearch = NULL;
  gchar *pRefProductSearch = NULL;
  gchar *pPrice = NULL;
  gchar *pNewPrice = NULL;
  gint i;
  gint quantity;
  gint newQuantity;
  gboolean foundProduct;
  
  listSubCategorySelected = gtk_tree_selection_get_selected_rows(pGlda->selectCategory, NULL);
  listProductsSelected = gtk_tree_selection_get_selected_rows(pGlda->selectProducts, NULL);
  
  if(listProductsSelected != NULL){
  	/* Get indices/path/iter of subcategory selected */
  	tabIndices = gtk_tree_path_get_indices(g_list_nth_data(listSubCategorySelected, 0));
  	pPath = gtk_tree_path_new_from_indices(tabIndices[0], tabIndices[1], -1);
  	gtk_tree_model_get_iter((GtkTreeModel*)pGlda->pTreeStoreCategory, &Iter, pPath);
  	/* Get references of category and subcategory */
  	gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreCategory, &Iter,
  																		REF_CAT_COLUMN, &pRefCategory,
  																		REF_SUBCAT_COLUMN, &pRefSubCategory,
  																		-1);
		
	/* Get number of products selected to add */
	NbProductsSelected = gtk_tree_selection_count_selected_rows(pGlda->selectProducts);
	i=0;
	for(i=0;i<NbProductsSelected;i++){
		if(pProductName != NULL){
			g_free(pProductName);
			g_free(pProductPrice);
			g_free(pRefProduct);
			
			pProductName = NULL;
			pProductPrice = NULL;
			pRefProduct = NULL;
		}
		/* Get indices/path/iter of product selected */
		tabIndices = gtk_tree_path_get_indices(g_list_nth_data(listProductsSelected, i));
		pPath = gtk_tree_path_new_from_indices(tabIndices[0], -1);
		gtk_tree_model_get_iter((GtkTreeModel*)pGlda->pTreeStoreProducts, &Iter, pPath);
		/* Get name/price/reference of product selected */
		gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreProducts, &Iter,
																			PRODUCTS_COLUMN, &pProductName,
																			PRICE_PRODUCTS_COLUMN, &pProductPrice,
																			REF_PRODUCTS_COLUMN, &pRefProduct,
																			-1);
		
		/* Get the first product added if exist */
		if(gtk_tree_model_get_iter_first((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter)){
			foundProduct = FALSE;
			do{
				/* Get references of product added */
				gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter,
								REF_CAT_PRODUCTS_ADDED_COLUMN, &pRefCategorySearch,
								REF_SUBCAT_PRODUCTS_ADDED_COLUMN, &pRefSubCategorySearch,
								REF_PRODUCTS_ADDED_COLUMN, &pRefProductSearch,
								-1);
				
				/* compare references */
				if(strcmp(pRefCategorySearch, pRefCategory) == 0 &&
					strcmp(pRefSubCategorySearch, pRefSubCategory) == 0 &&
					strcmp(pRefProductSearch, pRefProduct) == 0)
				{
					foundProduct = TRUE;
					/* Get actual price and quantity of product */
					gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter,
										PRICE_PRODUCTS_ADDED_COLUMN, &pPrice,
										QUANTITY_PRODUCTS_ADDED_COLUMN, &quantity,
										-1);
					pNewPrice = g_strdup_printf("%.2f",
												strtod(pPrice, NULL)+strtod(pProductPrice, NULL));
					newQuantity = quantity + 1;
					/* Set new price and quantity of product */
					gtk_list_store_set(pGlda->pTreeStoreProductsAdded, &Iter,
										PRICE_PRODUCTS_ADDED_COLUMN, pNewPrice,
										QUANTITY_PRODUCTS_ADDED_COLUMN, newQuantity,
										-1);
					
					/* Update number total of products */
					pGlda->nbTotalProducts++;
					
					/* Update price total of products */
					pGlda->priceTotalProducts = pGlda->priceTotalProducts
													+ strtod(pProductPrice, NULL);
					
					refreshTotalPriceAndQuantity(pGlda);
					
					g_free(pPrice);
					g_free(pNewPrice);
					pPrice = NULL;
					pNewPrice = NULL;
				}
				g_free(pRefCategorySearch);
				g_free(pRefSubCategorySearch);
				g_free(pRefProductSearch);
				pRefCategorySearch = NULL;
				pRefSubCategorySearch = NULL;
				pRefProductSearch = NULL;
			}while(!foundProduct && 
					gtk_tree_model_iter_next((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter));
			
			if(!foundProduct){
				/* Add new product */
				addProductToListProductsAdd(pGlda, pProductName, pProductPrice, pRefCategory, pRefSubCategory, pRefProduct, 1);
			}
		}else{
			/* Add new product */
			addProductToListProductsAdd(pGlda, pProductName, pProductPrice, pRefCategory, pRefSubCategory, pRefProduct, 1);
		}
	}
	
	g_free(pProductName);
	g_free(pProductPrice);
	g_free(pRefProduct);
	g_free(pRefCategory);
	g_free(pRefSubCategory);
	g_list_free(listSubCategorySelected);
	g_list_free(listProductsSelected);
  }
}

void addProductToListProductsAdd(
	glda *pGlda,
	gchar *pProductName, gchar *pProductPrice,
	gchar *pRefCategory, gchar *pRefSubCategory, gchar *pRefProduct,
	gint quantity)
{
	GtkTreeIter pIter;
	
	gtk_list_store_append(pGlda->pTreeStoreProductsAdded, &pIter);
	gtk_list_store_set(pGlda->pTreeStoreProductsAdded, &pIter,
  						PRODUCTS_ADDED_COLUMN, pProductName,
  						PRICE_PRODUCTS_ADDED_COLUMN, pProductPrice,
  						QUANTITY_PRODUCTS_ADDED_COLUMN, quantity,
  						REF_CAT_PRODUCTS_ADDED_COLUMN, pRefCategory,
  						REF_SUBCAT_PRODUCTS_ADDED_COLUMN, pRefSubCategory,
  						REF_PRODUCTS_ADDED_COLUMN, pRefProduct,
  						-1);
	
	pGlda->nbTotalProducts = pGlda->nbTotalProducts + quantity;
	pGlda->priceTotalProducts = pGlda->priceTotalProducts + strtod(pProductPrice, NULL);
	
	refreshTotalPriceAndQuantity(pGlda);
}

void refreshTotalPriceAndQuantity(glda *pGlda){
	gchar *pText = NULL;
	
	/* Update number total of products */
	pText = g_strdup_printf("%d", pGlda->nbTotalProducts);
	if(g_key_file_get_boolean(pGlda->pConfig, "product", "show_total_quantity", NULL)){
		gtk_label_set_label(GTK_LABEL(pGlda->pLabelNumberTotalProducts), pText);
		g_free(pText);
	}
	
	/* Update price total of products */
	pText = g_strdup_printf("%.2f %s", pGlda->priceTotalProducts,
																			g_key_file_get_string(pGlda->pConfig, "general", "devise", NULL));
	if(g_key_file_get_boolean(pGlda->pConfig, "product", "show_total_price", NULL)){
		gtk_label_set_label(GTK_LABEL(pGlda->pLabelPriceTotalProducts), pText);
		g_free(pText);
	}
	
	/* not save */
	pGlda->isSave = FALSE;
	if(pGlda->filesave)
		pText = g_strdup_printf("*%s - Glda", pGlda->filesave);
	else
		pText = g_strdup_printf("*newfile.lda - Glda");
	gtk_window_set_title(GTK_WINDOW(pGlda->pWindow), pText);
	g_free(pText);
}

void saveListProductsAdded(glda *pGlda){
	GtkWidget *pFileSelection;
	gboolean quit;
	gint result;
	
	if(pGlda->filesave != NULL){
		saveListProductsAddedToXml(pGlda);
	}else{
		pFileSelection = gtk_file_chooser_dialog_new("Enregistrer sous...",
													GTK_WINDOW(pGlda->pWindow),
													GTK_FILE_CHOOSER_ACTION_SAVE,
													GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
													GTK_STOCK_SAVE, GTK_RESPONSE_OK,
													NULL);
		
		quit = FALSE;
		while(!quit){
			result = gtk_dialog_run(GTK_DIALOG(pFileSelection));
			if(result == GTK_RESPONSE_OK){
				pGlda->filesave = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pFileSelection));
				if(g_file_test(pGlda->filesave, G_FILE_TEST_EXISTS) == TRUE){
					/* erase the exist file */
					if(dialogYesAndNo(pGlda, "Ce fichier existe déjà. Voulez-vous le remplacer ?")){
						saveListProductsAddedToXml(pGlda);
						quit = TRUE;
					}else{
						pGlda->filesave = NULL;
					}
				}else{
					saveListProductsAddedToXml(pGlda);
					quit = TRUE;
				}
			}else{
				quit = TRUE;
			}
		}
		gtk_widget_destroy(pFileSelection);
	}
}

/* save the list to xml */
gboolean saveListProductsAddedToXml(glda *pGlda){
	xmlDocPtr xmlDocListProductsAdded = NULL;
	xmlNodePtr root = NULL;
	xmlNodePtr productNode = NULL;
	xmlNodePtr NextProductNode = NULL;
	GtkTreeIter Iter;
  gchar *pRefCategory = NULL;
  gchar *pRefSubCategory = NULL;
  gchar *pRefProduct = NULL;
  gchar *pQuantity = NULL;
  /*gchar *pText = NULL;*/
  gchar *windowTitle = NULL;
  gint quantity;
  
	xmlDocListProductsAdded = xmlNewDoc((const xmlChar*)"1.0");
  root = xmlNewNode(NULL, (const xmlChar*)"lda");
  productNode = xmlNewNode(NULL, (const xmlChar*)"product");
  
  if(pGlda->nbTotalProducts == 0){
  	printf("Votre liste d'achat est encore vide\n");
  }
  
  xmlAddChild((xmlNodePtr)xmlDocListProductsAdded, root);
  
  /* Get the first product added if exist */
		if(gtk_tree_model_get_iter_first((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter)){
			/* Get references and quantity of product added */
			gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter,
																				QUANTITY_PRODUCTS_ADDED_COLUMN, &quantity,
																				REF_CAT_PRODUCTS_ADDED_COLUMN, &pRefCategory,
																				REF_SUBCAT_PRODUCTS_ADDED_COLUMN, &pRefSubCategory,
																				REF_PRODUCTS_ADDED_COLUMN, &pRefProduct,
																				-1);
			
			pQuantity = g_strdup_printf("%d", quantity);
			
			/* Add first product */
			xmlAddChild(root, productNode);
			xmlSetProp(productNode, (const xmlChar*)"refCategory", (const xmlChar*)pRefCategory);
			xmlSetProp(productNode, (const xmlChar*)"refSubCategory", (const xmlChar*)pRefSubCategory);
			xmlSetProp(productNode, (const xmlChar*)"refProduct", (const xmlChar*)pRefProduct);
			xmlSetProp(productNode, (const xmlChar*)"quantity", (const xmlChar*)pQuantity);
			
			g_free(pRefCategory);
			g_free(pRefSubCategory);
			g_free(pRefProduct);
			g_free(pQuantity);
			pRefCategory = NULL;
			pRefSubCategory = NULL;
			pRefProduct = NULL;
			pQuantity = NULL;
			
			while(gtk_tree_model_iter_next((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter)){
				/* Get references and quantity of product added */
				gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter,
								QUANTITY_PRODUCTS_ADDED_COLUMN, &quantity,
								REF_CAT_PRODUCTS_ADDED_COLUMN, &pRefCategory,
								REF_SUBCAT_PRODUCTS_ADDED_COLUMN, &pRefSubCategory,
								REF_PRODUCTS_ADDED_COLUMN, &pRefProduct,
								-1);
				
				pQuantity = g_strdup_printf("%d", quantity);
				
				NextProductNode = xmlNewNode(NULL, (const xmlChar*)"product");
				
				xmlAddSibling(productNode, NextProductNode);
				xmlSetProp(NextProductNode, (const xmlChar*)"refCategory", (const xmlChar*)pRefCategory);
				xmlSetProp(NextProductNode, (const xmlChar*)"refSubCategory", (const xmlChar*)pRefSubCategory);
				xmlSetProp(NextProductNode, (const xmlChar*)"refProduct", (const xmlChar*)pRefProduct);
				xmlSetProp(NextProductNode, (const xmlChar*)"quantity", (const xmlChar*)pQuantity);
				
				g_free(pRefCategory);
				g_free(pRefSubCategory);
				g_free(pRefProduct);
				g_free(pQuantity);
				pRefCategory = NULL;
				pRefSubCategory = NULL;
				pRefProduct = NULL;
				pQuantity = NULL;
				
				productNode = NextProductNode;
			}
		}
	
	/* save to file */
	if(xmlSaveFormatFileEnc(pGlda->filesave, xmlDocListProductsAdded, "UTF-8", 1) == -1){
    xmlFreeDoc(xmlDocListProductsAdded);
    return FALSE;
  }
  xmlFreeDoc(xmlDocListProductsAdded);
  
  windowTitle = g_strdup_printf("%s - Glda", pGlda->filesave);
  gtk_window_set_title(GTK_WINDOW(pGlda->pWindow), windowTitle);
  g_free(windowTitle);
  pGlda->isSave =TRUE;
  return TRUE;
}

void openListProducts(glda *pGlda){
	GtkWidget *pFileSelection;
	gchar *fileToOpen = NULL;
	gchar *pText = NULL;
	
	pFileSelection = gtk_file_chooser_dialog_new("Ouvrir...",
					       GTK_WINDOW(pGlda->pWindow),
					       GTK_FILE_CHOOSER_ACTION_OPEN,
					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
					       NULL);
	
	switch(gtk_dialog_run(GTK_DIALOG(pFileSelection))){
		case GTK_RESPONSE_OK:
			fileToOpen = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pFileSelection));
			if(g_file_test(fileToOpen, G_FILE_TEST_EXISTS) != TRUE){
				pText = g_strdup_printf("%s don't exist! :(", fileToOpen);
				dialogError(pGlda, pText);
				g_free(pText);
			}else{
				pGlda->filesave = g_strdup(fileToOpen);
				addListProductFromXml(pGlda);
			}
			break;
		default:
			break;
	}
	gtk_widget_destroy(pFileSelection);
}

gboolean addListProductFromXml(glda *pGlda){
	xmlDocPtr xmlDocListProductsAdded = NULL;
	xmlNodePtr root = NULL;
	
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const char path_template[] = "/lda/product[%d]";
  xmlChar *path;
	
	gchar *pRefCategory = NULL;
	gchar *pRefSubCategory = NULL;
	gchar *pRefProduct = NULL;
	gchar *pQuantity = NULL;
	
	gchar *pProductName = NULL;
	gchar *pProductPrice = NULL;
	gchar *pProductPriceTotal = NULL;
	
	gint i;
	gint nbProductNoFound;
	gint quantity;
	
	gint positionCategory;
	gint positionSubCategory;
	gint positionProduct;
	
	gchar *windowTitle = NULL;
	gchar *pText = NULL;
	
	xmlDocListProductsAdded = xmlParseFile(pGlda->filesave);
	
	if(xmlDocListProductsAdded == NULL){
    fprintf(stderr, "%s isn't a valide XML file.\n", pGlda->filesave);
    return FALSE;
  }
  
  root = xmlDocGetRootElement(xmlDocListProductsAdded);
  if(root == NULL){
    fprintf(stderr, "%s is empty.\n", pGlda->filesave);
    xmlFreeDoc(xmlDocListProductsAdded);
    return FALSE;
  }
  
  if(strcmp((char*)root->name, "lda") != 0){
    fprintf(stderr, "%s isn't a lda file.\n", pGlda->filesave);
    xmlFreeDoc(xmlDocListProductsAdded);
    return FALSE;
  }
  
  /* Get references and quantity of product */
  i = 1;
  nbProductNoFound = 0;
  xmlperms_context = xmlXPathNewContext(xmlDocListProductsAdded);
  
  path = (xmlChar*)g_strdup_printf(path_template, i);
  xmlobject = xmlXPathEval(path, xmlperms_context);
  
  /* Clear the list of products added */
	gtk_list_store_clear(pGlda->pTreeStoreProductsAdded);
	pGlda->nbTotalProducts = 0;
	pGlda->priceTotalProducts = 0;
	
  while(xmlobject->nodesetval && xmlobject->nodesetval->nodeTab){
  	pRefCategory = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "refCategory");
    pRefSubCategory = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "refSubCategory");
    pRefProduct = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "refProduct");
    pQuantity = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "quantity");
    
    positionCategory = getPositionCategoryByReference(pGlda->xmlDocListProducts, pRefCategory);
    if(positionCategory != -1){
    	positionSubCategory = getPositionSubCategoryByReference(pGlda->xmlDocListProducts,
    																													positionCategory,
    																													pRefSubCategory);
    	if(positionSubCategory != -1){
    		positionProduct = getPositionProductByReference(pGlda->xmlDocListProducts,
    																										positionCategory,
    																										positionSubCategory,
    																										pRefProduct);
    		if(positionProduct != -1){
    			getProductNameByPosition(pGlda->xmlDocListProducts,
    																positionCategory, positionSubCategory, positionProduct, &pProductName);
					getProductPriceByPosition(pGlda->xmlDocListProducts,
																		positionCategory, positionSubCategory, positionProduct, &pProductPrice);
					
    			quantity = strtol(pQuantity, NULL, 10);
    			pProductPriceTotal = g_strdup_printf("%.2f", strtod(pProductPrice, NULL)*quantity);
    			
    			addProductToListProductsAdd(pGlda, pProductName, pProductPriceTotal,
    																				 pRefCategory, pRefSubCategory, pRefProduct,
    																				 quantity);
					
    			g_free(pProductName);
    			g_free(pProductPrice);
    			g_free(pProductPriceTotal);
    			pProductName = NULL;
    			pProductPrice = NULL;
    			pProductPriceTotal = NULL;
    		}else{
    			nbProductNoFound++;
    		}
    	}else{
    		nbProductNoFound++;
    	}
    }else{
    	nbProductNoFound++;
    }
    
    g_free(pRefCategory);
    g_free(pRefSubCategory);
    g_free(pRefProduct);
    g_free(pQuantity);
    
    pRefCategory = NULL;
    pRefSubCategory = NULL;
    pRefProduct = NULL;
    pQuantity = NULL;
    
    g_free(path);
    xmlXPathFreeObject(xmlobject);
    
    i++;
    
    path = NULL;
    xmlobject = NULL;
    
    /* next product */
    path = (xmlChar*)g_strdup_printf(path_template, i);
  	xmlobject = xmlXPathEval(path, xmlperms_context);
  }
  
  pGlda->isSave = TRUE;
    			
	windowTitle = g_strdup_printf("%s - Glda", pGlda->filesave);
	gtk_window_set_title(GTK_WINDOW(pGlda->pWindow), windowTitle);
	g_free(windowTitle);
	
	if(nbProductNoFound > 0){
		pText = g_strdup_printf("%d article(s) est (sont) introuvable(s)", nbProductNoFound);
		dialogError(pGlda, pText);
		g_free(pText);
	}
	
  return TRUE;
}

/* return TRUE if quit */
gboolean saveOrNoBeforeQuit(glda *pGlda){
	GtkWidget *pQuestion;
  GtkWidget *pLabel;
  
  pQuestion = gtk_dialog_new_with_buttons("Quitter Glda",
					  	GTK_WINDOW(pGlda->pWindow),
						  GTK_DIALOG_MODAL,
						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						  GTK_STOCK_NO, GTK_RESPONSE_NO,
						  GTK_STOCK_YES, GTK_RESPONSE_YES,
						  NULL);
	
	pLabel = gtk_label_new("Voulez-vous enregistrer les modifications ?");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pQuestion)->vbox), pLabel, TRUE, FALSE, 0);
  
  gtk_widget_show_all(GTK_DIALOG(pQuestion)->vbox);
  
	switch(gtk_dialog_run(GTK_DIALOG(pQuestion))){
  /* save after quit */
  case GTK_RESPONSE_YES:
    gtk_widget_destroy(pQuestion);
    printf("Save and quit...\n");
    return TRUE;
    break;
  /* quit without save */
  case GTK_RESPONSE_NO:
    gtk_widget_destroy(pQuestion);
    printf("Not save and quit...\n");
    return TRUE;
    break;
  /* cancel. Not save not quit */
  case GTK_RESPONSE_CANCEL:
  default:
    gtk_widget_destroy(pQuestion);
    printf("Not save, not quit...\n");
    return FALSE;
    break;
  }
}

void dialogInfo(glda *pGlda, gchar *message){
	GtkWidget *pInfo;
	
	pInfo = gtk_message_dialog_new(GTK_WINDOW(pGlda->pWindow),
						GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
						GTK_BUTTONS_CLOSE,
						message);
	
	gtk_dialog_run(GTK_DIALOG(pInfo));
	
	gtk_widget_destroy(pInfo);
}

gboolean dialogYesAndNo(glda *pGlda, gchar *message){
	GtkWidget *pQuestion;
  
  pQuestion = gtk_message_dialog_new(GTK_WINDOW(pGlda->pWindow),
				     GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
				     GTK_BUTTONS_YES_NO,
				     message);
  
  switch(gtk_dialog_run(GTK_DIALOG(pQuestion))){
  case GTK_RESPONSE_YES:
    gtk_widget_destroy(pQuestion);
    return TRUE;
    break;
  case GTK_RESPONSE_NO:
    gtk_widget_destroy(pQuestion);
    return FALSE;
    break;
  default:
    gtk_widget_destroy(pQuestion);
    return FALSE;
    break;
  }
}

void dialogError(glda *pGlda, gchar *message){
	GtkWidget *pError;
	
	pError = gtk_message_dialog_new(GTK_WINDOW(pGlda->pWindow),
				  	GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
				  	GTK_BUTTONS_CLOSE,
				  	message);
	
	gtk_dialog_run(GTK_DIALOG(pError));
  
  gtk_widget_destroy(pError);
}


/********************************* About's Window *********************************/

void create_pAbout(glda *pGlda){
  GtkWidget *pAbout;
  GdkPixbuf *logo;
  /*GError *error = NULL;*/
  
  const gchar *authors[] = {
    "Tuxgasy <tuxgasy@tuxfamily.org>",
    NULL
  };
  
  const gchar *documenters[] = {
    "fanfan la tulipe <olivier.fandard1@tele2.fr>",
    NULL
  };
  
  const gchar *artists[] = {
    "Isabelle Stévant <isabelle.stevant@gmail.com>",
    NULL
  };
  
  gchar *license = NULL;
  gchar *tmp = NULL;
  FILE *file_license = NULL;
  
  file_license = fopen(LICENSE_FILE, "r");
  if(file_license){
    license = (gchar*)malloc(sizeof(char)*920);
    tmp = (gchar*)malloc(sizeof(char)*40);
    strcpy(license, "");
    while(fgets(tmp, 40, file_license)){
      strcat(license, tmp);
    }
    free(tmp);
    fclose(file_license);
  }
  else{
    license = g_strdup_printf("GPL version 3 ou superieure\n");
  }
  
  pAbout = gtk_about_dialog_new();
  
  logo = gdk_pixbuf_new_from_file(ICON_GLDA, NULL);
  
  gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(pAbout), logo);
  
  gtk_window_set_icon_from_file(GTK_WINDOW(pAbout), ICON_GLDA, NULL);
  
  /*gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(pAbout), "À propos de Glda");*/
  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(pAbout), VERSION_GLDA);
  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(pAbout), "(c) 2008 Tuxgasy");
  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(pAbout),
				"Glda (Gtk Liste D'Achat) est un utilitaire pour faire sa liste d'achat.");
  
  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(pAbout), (const gchar*)license);
  free(license);
  
  gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(pAbout), "http://glda.tuxfamily.org");
  
  gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(pAbout), authors);
  gtk_about_dialog_set_documenters(GTK_ABOUT_DIALOG(pAbout), documenters);
  gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(pAbout), artists);
  
  gtk_dialog_run(GTK_DIALOG(pAbout));
  gtk_widget_destroy(pAbout);
}
