/*
 *      preferences.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "main.h"
#include "listProducts.h"
#include "interface.h"
#include "callbacks.h"
#include "config.h"
#include "preferences.h"
#include "print.h"

/********************************** Interface **********************************/

void create_pPreferences(glda *pGlda){
	GtkWidget *pPreferences;
  GtkWidget *pNotebook;
  GtkWidget *pGeneral;
  GtkWidget *pPrint;
  GtkWidget *pListProducts;
  GtkWidget *pTabLabel;
  Preferences *pPreferencesData;
  gchar *fileUserConfig;
  gchar *filename;
  
  /* init Data */
  pPreferencesData = (Preferences *) malloc(sizeof(Preferences));
  pPreferencesData->pGlda = pGlda;
  pPreferencesData->isNeedReboot = FALSE;
  pPreferencesData->isNeedReloadList = FALSE;
  
	pPreferences = gtk_dialog_new_with_buttons("Préférences", GTK_WINDOW(pGlda->pWindow),
					     GTK_DIALOG_MODAL,
					     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
					     NULL);
	
	/* Notebook */
	pNotebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pPreferences)->vbox), pNotebook, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(pNotebook), GTK_POS_TOP);
  gtk_notebook_set_scrollable(GTK_NOTEBOOK(pNotebook), TRUE);
	
	/*** General's tab ***/
  pGeneral = create_pGeneral(pPreferencesData);
  pTabLabel = gtk_label_new("Général");
  gtk_notebook_append_page(GTK_NOTEBOOK(pNotebook), pGeneral, pTabLabel);
  
  /*** Print's tab ***/
  pPrint = create_pPrintPreferences(pPreferencesData);
  pTabLabel = gtk_label_new("Impression");
  gtk_notebook_append_page(GTK_NOTEBOOK(pNotebook), pPrint, pTabLabel);
  
  /*** List of products'tab ***/
  pListProducts = create_pListProducts(pPreferencesData);
  pTabLabel = gtk_label_new("Liste d'articles");
  gtk_notebook_append_page(GTK_NOTEBOOK(pNotebook), pListProducts, pTabLabel);
  
	/*  */
	gtk_widget_show_all(GTK_DIALOG(pPreferences)->vbox);
	
	switch(gtk_dialog_run(GTK_DIALOG(pPreferences))){
	  case GTK_RESPONSE_CLOSE:
  	default:
  		gtk_widget_destroy(pPreferences);
  		fileUserConfig = g_strdup_printf("%s%s%s", getenv("HOME"), DIR_USER_CONFIG, FILE_USER_CONFIG);
  		saveConfig(pPreferencesData->pGlda->pConfig, fileUserConfig);
  		g_free(fileUserConfig);
  		if(pPreferencesData->isNeedReboot)
  			dialogInfo(pPreferencesData->pGlda,
  								"Certaine(s) modification(s) nécessite(nt) de relancer l'application pour être prise en compte");
  		if(pPreferencesData->isNeedReloadList){
  			/* save list of products on file */
  			filename = g_key_file_get_string(pPreferencesData->pGlda->pConfig,
  																					"file", "file_list_product", NULL);
  			xmlSaveFormatFileEnc(filename, pPreferencesData->pGlda->xmlDocListProducts, "UTF-8", 1);
  			g_free(filename);
  			/* reload list of products */
  			gtk_list_store_clear(pPreferencesData->pGlda->pTreeStoreProducts);
  			gtk_tree_store_clear(pPreferencesData->pGlda->pTreeStoreCategory);
  			create_ListCategory(pPreferencesData->pGlda);
  		}
  		break;
  }
}

GtkWidget *create_pGeneral(Preferences *pPreferencesData){
	GtkWidget *pGeneral;
  GtkWidget *pLabel;
  GtkWidget *pEntry;
  GtkWidget *pCheckButton;
  GtkWidget *pHBox;
  gchar *pText;
  gboolean keyBool;
  
  pGeneral = gtk_vbox_new(FALSE, 10);
  
  pLabel = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(pLabel), _("<b>Général</b>"));
  gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(pLabel), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(pGeneral), pLabel, FALSE, TRUE, 0);
  
  pHBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(pGeneral), pHBox, FALSE, TRUE, 0);
  
  pLabel = gtk_label_new("Devise : ");
  gtk_box_pack_start(GTK_BOX(pHBox), pLabel, FALSE, TRUE, 5);
  
  pEntry = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(pEntry), 5);
  pText = g_key_file_get_string(pPreferencesData->pGlda->pConfig, "general", "devise", NULL);
  gtk_entry_set_text(GTK_ENTRY(pEntry), pText);
  g_free(pText);
  g_signal_connect(G_OBJECT(pEntry), "changed", G_CALLBACK(OnChangedDevise), (Preferences*) pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pHBox), pEntry, FALSE, TRUE, 0);
  
  pLabel = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(pLabel), _("<b>Affichage</b>"));
  gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(pLabel), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(pGeneral), pLabel, FALSE, TRUE, 0);
  
  pCheckButton = gtk_check_button_new_with_label("Afficher la colonne prix");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "product", "show_column_price", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowColumnPriceProduct), (Preferences*) pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pGeneral), pCheckButton, FALSE, TRUE, 0);
  
  pCheckButton = gtk_check_button_new_with_label("Afficher la colonne quantité");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "product", "show_column_quantity", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowColumnQuantityProduct), (Preferences*) pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pGeneral), pCheckButton, FALSE, TRUE, 0);
  
  pCheckButton = gtk_check_button_new_with_label("Afficher le nombre total d'articles");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "product", "show_total_quantity", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowTotalQuantityProduct), (Preferences*) pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pGeneral), pCheckButton, FALSE, TRUE, 0);
  
  pCheckButton = gtk_check_button_new_with_label("Afficher le prix total des articles");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "product", "show_total_price", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowTotalPriceProduct), (Preferences*) pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pGeneral), pCheckButton, FALSE, TRUE, 0);
  
  return pGeneral;
}

GtkWidget *create_pPrintPreferences(Preferences *pPreferencesData){
	GtkWidget *pPrint;
	GtkWidget *pHBox;
	GtkWidget *pLabel;
	GtkWidget *pButton;
	GtkWidget *pCheckButton;
	GtkWidget *pSpin;
	gboolean keyBool;
	gint keyInteger;
	
	pPrint = gtk_vbox_new(FALSE, 10);
	
	pLabel = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(pLabel), _("<b>Polices</b>"));
  gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(pLabel), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(pPrint), pLabel, FALSE, TRUE, 0);
  
  pHBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(pPrint), pHBox, FALSE, TRUE, 0);
  
  pLabel = gtk_label_new("Police d'impression : ");
	gtk_box_pack_start(GTK_BOX(pHBox), pLabel, FALSE, TRUE, 0);
	
	pButton = gtk_button_new_with_label("Choisir une police");
	g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnSelectFontPrint), pPreferencesData);
	gtk_box_pack_start(GTK_BOX(pHBox), pButton, FALSE, TRUE, 0);
	
	pHBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(pPrint), pHBox, FALSE, TRUE, 0);
  
	pLabel = gtk_label_new("Couleur d'impression : ");
	gtk_box_pack_start(GTK_BOX(pHBox), pLabel, FALSE, TRUE, 0);
	
	pButton = gtk_button_new_with_label("Choisir une couleur");
	g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnSelectColorPrint), pPreferencesData);
	gtk_box_pack_start(GTK_BOX(pHBox), pButton, FALSE, TRUE, 0);
	
	pLabel = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(pLabel), _("<b>Mise en page</b>"));
	gtk_label_set_justify(GTK_LABEL(pLabel), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(pLabel), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(pPrint), pLabel, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher le titre");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_title", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowTitlePrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher le numéro de la page");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_page", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowPagePrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	pHBox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(pPrint), pHBox, FALSE, TRUE, 0);
  
  pLabel = gtk_label_new("Bordure : ");
  gtk_box_pack_start(GTK_BOX(pHBox), pLabel, FALSE, TRUE, 0);
  
	pSpin = gtk_spin_button_new_with_range(0, 5, 1);
	keyInteger = g_key_file_get_integer(pPreferencesData->pGlda->pConfig, "print", "border", NULL);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pSpin), (gdouble)keyInteger);
	g_signal_connect(G_OBJECT(pSpin), "value-changed",
																G_CALLBACK(OnValueChangedBorder), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pHBox), pSpin, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher la colonne numéro");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_number", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowColumnNumberPrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher la colonne nom");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_name", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowColumnNamePrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher la colonne prix");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_price", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowColumnPricePrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher la colonne quantité");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_quantity", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowColumnQuantityPrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	pCheckButton = gtk_check_button_new_with_label("Afficher la ligne total");
  keyBool = g_key_file_get_boolean(pPreferencesData->pGlda->pConfig, "print", "show_line_total", NULL);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pCheckButton), keyBool);
  g_signal_connect(G_OBJECT(pCheckButton), "clicked",
  									G_CALLBACK(OnClickedShowLineTotalPrint), (Preferences*) pPreferencesData);
	
	gtk_box_pack_start(GTK_BOX(pPrint), pCheckButton, FALSE, TRUE, 0);
	
	return pPrint;
}

void createFontSelectionDialog(Preferences *pPreferencesData){
	GtkWidget *pDialog;
	gchar *police = NULL;
	gdouble size_police;
	gchar *font_name = NULL;
	
	police = g_key_file_get_string(pPreferencesData->pGlda->pConfig, "print", "police", NULL);
	size_police = g_key_file_get_integer(pPreferencesData->pGlda->pConfig, "print", "size_police", NULL);
	
	font_name = g_strdup_printf("%s Regular %.2f", police, size_police);
	g_free(police);
	g_free(font_name);
	
	pDialog = gtk_font_selection_dialog_new("Choisissez une police");
	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(pDialog), font_name);
	
	gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(pDialog), "Glda est un outils de créer de liste d'achat");
	
	switch(gtk_dialog_run(GTK_DIALOG(pDialog))){
		case GTK_RESPONSE_OK:
			font_name = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(pDialog));
			printf("polices : %s\n", font_name);
			g_free(font_name);
			gtk_widget_destroy(pDialog);
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			gtk_widget_destroy(pDialog);
			break;
	}
}

void createColorSelectionDialog(Preferences *pPreferencesData){
	GtkWidget *pDialog, *colorsel;
	
	pDialog = gtk_color_selection_dialog_new("Selecteur de couleur");
	
	colorsel = GTK_COLOR_SELECTION_DIALOG(pDialog)->colorsel;
	
	
	switch(gtk_dialog_run(GTK_DIALOG(pDialog))){
		case GTK_RESPONSE_OK:
			
			gtk_widget_destroy(pDialog);
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			gtk_widget_destroy(pDialog);
			break;
	}
}

GtkWidget *create_pListProducts(Preferences *pPreferencesData){
	GtkWidget *pListProducts;
	GtkWidget *pFrame;
	GtkWidget *pScrollbar;
	GtkWidget *pButton;
	GtkWidget *pHBox;
	GtkWidget *pVBox;
	GtkWidget *pTable;
	GtkWidget *pEditListProducts;
	GtkWidget *pTreeView;
	
	GtkTreeViewColumn *pColumn;
  GtkCellRenderer *pCellRenderer;
  
	gchar *pText;
	
	pListProducts = gtk_vbox_new(FALSE, 0);
	
	/*** File with list of products ***/
	pFrame = gtk_frame_new("Fichier de liste d'articles");
  gtk_box_pack_start(GTK_BOX(pListProducts), pFrame, FALSE, TRUE, 0);
  
  pHBox = gtk_hbox_new(TRUE, 0);
  gtk_container_add(GTK_CONTAINER(pFrame), pHBox);
	
	pPreferencesData->pEntryListProducts = gtk_entry_new();
	pText = g_key_file_get_string(pPreferencesData->pGlda->pConfig, "file", "file_list_product", NULL);
	gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryListProducts), pText);
	gtk_entry_set_editable(GTK_ENTRY(pPreferencesData->pEntryListProducts), FALSE);
	gtk_box_pack_start(GTK_BOX(pHBox), pPreferencesData->pEntryListProducts, FALSE, TRUE, 0);
	g_free(pText);
	
	pButton = gtk_button_new_from_stock(GTK_STOCK_OPEN);
	gtk_box_pack_start(GTK_BOX(pHBox), pButton, FALSE, TRUE, 0);
	g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnSelectFileListProducts), pPreferencesData);
	
	pTable = gtk_table_new(1, 2, TRUE);
	gtk_box_pack_start(GTK_BOX(pListProducts), pTable, FALSE, TRUE, 0);
	
	/*** Edit list of products ***/
	pEditListProducts = create_EditListProducts(pPreferencesData);
	gtk_table_attach_defaults(GTK_TABLE(pTable), pEditListProducts, 0, 1, 0, 1);
	
	/*** List of products ***/
	pVBox = gtk_vbox_new(TRUE, 0);
	gtk_table_attach_defaults(GTK_TABLE(pTable), pVBox, 1, 2, 0, 1);
	
	pFrame = gtk_frame_new("Liste d'articles");
	gtk_box_pack_start(GTK_BOX(pVBox), pFrame, TRUE, TRUE, 0);
	
	pVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(pFrame), pVBox);
	
	pScrollbar = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(pVBox), pScrollbar, TRUE, TRUE, 0);
  
	pPreferencesData->pTreeListProducts = gtk_tree_store_new(N_COLUMN_PRODUCTS_PREFERENCES,
																					G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	createListProducts(pPreferencesData);
	pTreeView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pPreferencesData->pTreeListProducts));
	
	/* allow to search by name */
  gtk_tree_view_set_search_column(GTK_TREE_VIEW(pTreeView), COLUMN_PRODUCTS_PREFERENCES);
  
	/* single selection */
  pPreferencesData->selectListProducts = gtk_tree_view_get_selection(GTK_TREE_VIEW(pTreeView));
  gtk_tree_selection_set_mode(pPreferencesData->selectListProducts, GTK_SELECTION_SINGLE);
	
	pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Articles", pCellRenderer, "text", COLUMN_PRODUCTS_PREFERENCES, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_sort_column_id(pColumn, COLUMN_PRODUCTS_PREFERENCES);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
	
	pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Prix", pCellRenderer, "text", COLUMN_PRICE_PREFERENCES, NULL);
  gtk_tree_view_column_set_resizable(pColumn, TRUE);
  gtk_tree_view_column_set_sort_column_id(pColumn, COLUMN_PRICE_PREFERENCES);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  
	pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("refCat", pCellRenderer, "text", COLUMN_REFCATEGORY, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("refSubCat", pCellRenderer, "text", COLUMN_REFSUBCATEGORY, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("refProduct", pCellRenderer, "text", COLUMN_REFPRODUCTS, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);
  /* hide reference's column */
  gtk_tree_view_column_set_visible(pColumn, FALSE);
  
  gtk_container_add(GTK_CONTAINER(pScrollbar), pTreeView);
	
	pHBox = gtk_hbox_new(TRUE, 0);
	
	/* button delete */
  pButton = gtk_button_new_from_stock(GTK_STOCK_DELETE);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnRemoveFromList), (Preferences*)pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pHBox), pButton, TRUE, TRUE, 0);
  
  /* button edit */
  pButton = gtk_button_new_from_stock(GTK_STOCK_EDIT);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnEditList), (Preferences*)pPreferencesData);
  gtk_box_pack_start(GTK_BOX(pHBox), pButton, TRUE, TRUE, 0);
  
  gtk_box_pack_start(GTK_BOX(pVBox), pHBox, FALSE, FALSE, 0);
  
	return pListProducts;
}

GtkWidget *create_EditListProducts(Preferences *pPreferencesData){
	GtkWidget *pEditListProducts;
	GtkWidget *pFrame;
  GtkWidget *pTable;
  GtkWidget *pLabel;
  GtkWidget *pButton;
  
	pEditListProducts = gtk_vbox_new(FALSE, 0);
	
	/*** category ***/
	pFrame = gtk_frame_new("Catégorie");
  gtk_box_pack_start(GTK_BOX(pEditListProducts), pFrame, FALSE, TRUE, 0);
  
  pTable = gtk_table_new(2, 2, TRUE);
  
  pLabel = gtk_label_new("Nom :");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
  pPreferencesData->pEntryCategory = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pEntryCategory, 1, 2, 0, 1);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnClearCategory), (Preferences *)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pButton, 0, 1, 1, 2);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_OK);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnAddCategory), (Preferences*)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pButton, 1, 2, 1, 2);
  
  gtk_container_add(GTK_CONTAINER(pFrame), pTable);
  
  /*** subcategory ***/
  pFrame = gtk_frame_new("Sous-Catégorie");
  gtk_box_pack_start(GTK_BOX(pEditListProducts), pFrame, FALSE, TRUE, 0);
  
  pTable = gtk_table_new(3, 2, TRUE);
  
  pLabel = gtk_label_new("Catégorie : ");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
  
  pPreferencesData->pComboBoxCategoryInSubCategory = createComboBoxCategory(pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pComboBoxCategoryInSubCategory, 1, 2, 0, 1);
  
  pLabel = gtk_label_new("Nom :");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 1, 2);
  pPreferencesData->pEntrySubCategory = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pEntrySubCategory, 1, 2, 1, 2);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnClearSubCategory), (glda *)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pButton, 0, 1, 2, 3);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_OK);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnAddSubCategory), (Preferences*)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pButton, 1, 2, 2, 3);
  
  gtk_container_add(GTK_CONTAINER(pFrame), pTable);
  
  /*** product ***/
  pFrame = gtk_frame_new("Article");
  gtk_box_pack_start(GTK_BOX(pEditListProducts), pFrame, FALSE, TRUE, 0);
  
  pTable = gtk_table_new(5, 2, TRUE);
  
  pLabel = gtk_label_new("Catégorie : ");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
  
  pPreferencesData->pComboBoxCategoryInProduct = createComboBoxCategory(pPreferencesData);
  g_signal_connect(G_OBJECT(pPreferencesData->pComboBoxCategoryInProduct), "changed",
  									G_CALLBACK(OnChangedComboBoxCategory), (Preferences*)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pComboBoxCategoryInProduct, 1, 2, 0, 1);
  
  pLabel = gtk_label_new("Sous-catégorie : ");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 1, 2);
  
  pPreferencesData->pComboBoxSubCategoryInProduct = createComboBoxSubCategory(pPreferencesData, 1);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pComboBoxSubCategoryInProduct, 1, 2, 1, 2);
  
  pLabel = gtk_label_new("Nom :");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 2, 3);
  pPreferencesData->pEntryProductName = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pEntryProductName, 1, 2, 2, 3);
  
  pLabel = gtk_label_new("Prix :");
  gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 3, 4);
  pPreferencesData->pEntryProductPrice = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(pTable), pPreferencesData->pEntryProductPrice, 1, 2, 3, 4);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnClearProduct), (glda *)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pButton, 0, 1, 4, 5);
  
  pButton = gtk_button_new_from_stock(GTK_STOCK_OK);
  g_signal_connect(G_OBJECT(pButton), "clicked", G_CALLBACK(OnAddProduct), (Preferences*)pPreferencesData);
  gtk_table_attach_defaults(GTK_TABLE(pTable), pButton, 1, 2, 4, 5);
  
  gtk_container_add(GTK_CONTAINER(pFrame), pTable);
  
	return pEditListProducts;
}

void createListProducts(Preferences *pPreferencesData){
	GtkTreeIter pIterCategory;
  GtkTreeIter pIterSubCategory;
  GtkTreeIter pIterProduct;
	
	gchar *pCategory = NULL;
	gchar *pRefCategory = NULL;
	
  gchar *pSubCategory = NULL;
  gchar *pRefSubCategory = NULL;
  
  gchar *pProductName = NULL;
  gchar *pProductPrice = NULL;
  gchar *pRefProduct = NULL;
  
  gint positionCategory, positionSubCategory, positionProduct;
  gint errorCategory, errorSubCategory, errorProduct;
	
	/* Add category */
	positionCategory = 1;
	do{
  	if(pCategory != NULL){
  		g_free(pCategory);
  		g_free(pRefCategory);
  		pCategory = NULL;
  		pRefCategory = NULL;
  	}
  	errorCategory = getCategoryByPosition(pPreferencesData->pGlda->xmlDocListProducts,
  																				positionCategory, &pCategory, &pRefCategory);
  	if(errorCategory == CATEGORY_NO_ERROR){
  		gtk_tree_store_append(pPreferencesData->pTreeListProducts, &pIterCategory, NULL);
  		gtk_tree_store_set(pPreferencesData->pTreeListProducts, &pIterCategory,
  												COLUMN_PRODUCTS_PREFERENCES, pCategory,
  												COLUMN_PRICE_PREFERENCES, "",
  												COLUMN_REFCATEGORY, pRefCategory,
  												COLUMN_REFSUBCATEGORY, "",
  												COLUMN_REFPRODUCTS, "",
  												-1);
			
			/* Add subcategory */
			positionSubCategory = 1;
	  	do{
  			if(pSubCategory != NULL){
  				g_free(pSubCategory);
  				g_free(pRefSubCategory);
  				pSubCategory = NULL;
					pRefSubCategory = NULL;
				}
				errorSubCategory = getSubCategoryByPosition(pPreferencesData->pGlda->xmlDocListProducts,
  																									positionCategory, positionSubCategory,
  																									&pSubCategory, &pRefSubCategory);
				if(errorSubCategory == CATEGORY_NO_ERROR){
					gtk_tree_store_append(pPreferencesData->pTreeListProducts, &pIterSubCategory, &pIterCategory);
					gtk_tree_store_set(pPreferencesData->pTreeListProducts, &pIterSubCategory,
  													COLUMN_PRODUCTS_PREFERENCES, pSubCategory,
  													COLUMN_PRICE_PREFERENCES, "",
  													COLUMN_REFCATEGORY, pRefCategory,
  													COLUMN_REFSUBCATEGORY, pRefSubCategory,
  													COLUMN_REFPRODUCTS, "",
  												-1);
					
					/* Add products */
					positionProduct = 1;
					do{
						if(pProductName != NULL){
							g_free(pProductName);
							g_free(pProductPrice);
							g_free(pRefProduct);
							pProductName = NULL;
							pProductPrice = NULL;
							pRefProduct = NULL;
						}
						errorProduct = getProductReferenceByPosition(pPreferencesData->pGlda->xmlDocListProducts,
																			positionCategory, positionSubCategory, positionProduct, &pRefProduct);
						getProductNameByPosition(pPreferencesData->pGlda->xmlDocListProducts,
																			positionCategory, positionSubCategory, positionProduct, &pProductName);
						getProductPriceByPosition(pPreferencesData->pGlda->xmlDocListProducts,
																			positionCategory, positionSubCategory, positionProduct, &pProductPrice);
						
						if(errorProduct == PRODUCT_NO_ERROR){
							gtk_tree_store_append(pPreferencesData->pTreeListProducts, &pIterProduct, &pIterSubCategory);
							gtk_tree_store_set(pPreferencesData->pTreeListProducts, &pIterProduct,
  															COLUMN_PRODUCTS_PREFERENCES, pProductName,
  															COLUMN_PRICE_PREFERENCES, pProductPrice,
  															COLUMN_REFCATEGORY, pRefCategory,
  															COLUMN_REFSUBCATEGORY, pRefSubCategory,
		  													COLUMN_REFPRODUCTS, pRefProduct,
  															-1);
						}
						
						positionProduct++;
					}while(errorProduct == PRODUCT_NO_ERROR);
				}
				
				positionSubCategory++;
			}while(errorSubCategory == CATEGORY_NO_ERROR);
  	}
  	positionCategory++;
  }while(errorCategory == CATEGORY_NO_ERROR);
}

GtkWidget *createComboBoxCategory(Preferences *pPreferencesData){
	GtkWidget *pCombo;
  gint position;
  gint error;
  gchar *pCategory = NULL;
  
  pCombo = gtk_combo_box_new_text();
  
  position = 1;
  do{
  	if(pCategory != NULL){
  		g_free(pCategory);
  		pCategory = NULL;
  	}
  	error = getCategoryByPosition(pPreferencesData->pGlda->xmlDocListProducts,
  																				position, &pCategory, NULL);
  	if(error == CATEGORY_NO_ERROR){
  		gtk_combo_box_append_text(GTK_COMBO_BOX(pCombo), pCategory);
  	}
  	position++;
  }while(error == CATEGORY_NO_ERROR);
  
  gtk_combo_box_set_active(GTK_COMBO_BOX(pCombo), 0);
  
  return pCombo;
}

GtkWidget *createComboBoxSubCategory(Preferences *pPreferencesData, gint positionCategory){
	GtkWidget *pCombo;
	gint position;
	gint error;
	gchar *pSubCategory = NULL;
	
	pCombo = gtk_combo_box_new_text();
	
	position = 1;
	do{
		if(pSubCategory != NULL){
			g_free(pSubCategory);
			pSubCategory = NULL;
		}
		error = getSubCategoryByPosition(pPreferencesData->pGlda->xmlDocListProducts,
																				positionCategory, position, &pSubCategory, NULL);
		if(error == CATEGORY_NO_ERROR){
			gtk_combo_box_append_text(GTK_COMBO_BOX(pCombo), pSubCategory);
		}
		position++;
	}while(error == CATEGORY_NO_ERROR);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(pCombo), 0);
	
	return pCombo;
}

void changeComboBoxSubCategory(Preferences *pPreferencesData, gint positionCategory){
	gint position;
	gint error;
	gchar *pSubCategory = NULL;
	
	/* clear combo box */
	gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), 0);
	position = gtk_combo_box_get_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct));
	
	while(position != -1){
		gtk_combo_box_remove_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), position);
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), 0);
		position = gtk_combo_box_get_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct));
	}
	
	/* create new combo box */
	if(positionCategory == 0){
		return;
	}
	position = 1;
	do{
		if(pSubCategory != NULL){
			g_free(pSubCategory);
			pSubCategory = NULL;
		}
		error = getSubCategoryByPosition(pPreferencesData->pGlda->xmlDocListProducts,
																				positionCategory, position, &pSubCategory, NULL);
		if(error == CATEGORY_NO_ERROR){
			gtk_combo_box_append_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), pSubCategory);
		}
		position++;
	}while(error == CATEGORY_NO_ERROR);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), 0);
}

void removeFromList(Preferences *pPreferencesData){
	GtkWidget *pDialog;
	GtkTreePath *pPath;
	GtkTreeIter Iter;
	GList *list;
	gint depth;
	gint *tabIndices = NULL;
	gchar *pRefCategory = NULL;
	gchar *pRefSubCategory = NULL;
	gchar *pRefProduct = NULL;
	gchar *pText = NULL;
	
	list = gtk_tree_selection_get_selected_rows(pPreferencesData->selectListProducts, NULL);
	
	if(list != NULL){
		depth = gtk_tree_path_get_depth(list->data);
		tabIndices = gtk_tree_path_get_indices(list->data);
		
		switch(depth){
			/* category selected */
			case 1:
				pText = g_strdup_printf("Voulez-vous supprimer la catégorie sélectionnée avec les sous-categories et les articles liés?");
				pPath = gtk_tree_path_new_from_indices(tabIndices[0], -1);
				gtk_tree_model_get_iter((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter, pPath);
				gtk_tree_model_get((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter,
  																				COLUMN_REFCATEGORY, &pRefCategory,
  																				-1);
				break;
			/* subcategory selected */
			case 2:
				pText = g_strdup_printf("Voulez-vous supprimer la sous-catégorie sélectionnée avec les articles liés?");
				pPath = gtk_tree_path_new_from_indices(tabIndices[0], tabIndices[1], -1);
				gtk_tree_model_get_iter((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter, pPath);
				gtk_tree_model_get((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter,
  																				COLUMN_REFCATEGORY, &pRefCategory,
  																				COLUMN_REFSUBCATEGORY, &pRefSubCategory,
  																				-1);
				break;
			/* product selected */
			case 3:
				pText = g_strdup_printf("Voulez-vous supprimer l'article sélectionné?");
				pPath = gtk_tree_path_new_from_indices(tabIndices[0], tabIndices[1], tabIndices[2], -1);
				gtk_tree_model_get_iter((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter, pPath);
				gtk_tree_model_get((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter,
  																				COLUMN_REFCATEGORY, &pRefCategory,
  																				COLUMN_REFSUBCATEGORY, &pRefSubCategory,
  																				COLUMN_REFPRODUCTS, &pRefProduct,
  																				-1);
				break;
			default:
				break;
		}
		
		pDialog = gtk_message_dialog_new(GTK_WINDOW(pPreferencesData->pGlda->pWindow),
				     GTK_DIALOG_MODAL,
				     GTK_MESSAGE_QUESTION,
				     GTK_BUTTONS_YES_NO,
				     pText);
		g_free(pText);
		
		switch(gtk_dialog_run(GTK_DIALOG(pDialog))){
			case GTK_RESPONSE_YES:
      	gtk_widget_destroy(pDialog);
      	pPreferencesData->isNeedReloadList = TRUE;
      	switch(depth){
      		case 1:
      			removeCategoryFromInterface(pPreferencesData, pRefCategory);
      			gtk_tree_store_remove(pPreferencesData->pTreeListProducts, &Iter);
      			removeCategoryFromListProducts(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory);
      			break;
      		case 2:
      			removeSubCategoryFromInterface(pPreferencesData, pRefCategory, pRefSubCategory);
      			gtk_tree_store_remove(pPreferencesData->pTreeListProducts, &Iter);
      			removeSubCategoryFromListProducts(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory, pRefSubCategory);
      			break;
      		case 3:
      			removeProductFromListProducts(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory, pRefSubCategory, pRefProduct);
      			gtk_tree_store_remove(pPreferencesData->pTreeListProducts, &Iter);
      			break;
      		default:
      			break;
      	}
      	break;
      case GTK_RESPONSE_NO:
      default:
      	gtk_widget_destroy(pDialog);
      	break;
		}
		
		if(pRefCategory)
			g_free(pRefCategory);
		if(pRefSubCategory)
			g_free(pRefSubCategory);
		if(pRefProduct)
			g_free(pRefProduct);
	}
}

void removeCategoryFromInterface(Preferences *pPreferencesData, gchar *pRef){
	gint position;
	
	position = getPositionCategoryByReference(pPreferencesData->pGlda->xmlDocListProducts, pRef);
	
	gtk_combo_box_remove_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInSubCategory), position-1);
	gtk_combo_box_remove_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), position-1);
}

void removeSubCategoryFromInterface(Preferences *pPreferencesData, gchar *pRefCategory, gchar *pRef){
	gint positionCategory, position;
	
	positionCategory = getPositionCategoryByReference(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory);
	position = getPositionSubCategoryByReference(pPreferencesData->pGlda->xmlDocListProducts, positionCategory, pRef);
	
	gtk_combo_box_remove_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), position-1);
}

void editList(Preferences *pPreferencesData){
	GtkWidget *pDialog = NULL;
	GtkWidget *pLabel = NULL;
	GtkWidget *pEntry = NULL;
	GtkWidget *pEntryName = NULL;
	GtkWidget *pEntryPrice = NULL;
	GtkWidget *pTable;
	
	GtkTreePath *pPath;
	GtkTreeIter Iter;
	GList *list;
	gint depth;
	gint *tabIndices = NULL;
	gchar *pCategory = NULL;
	gchar *pRefCategory = NULL;
	gchar *pSubCategory = NULL;
	gchar *pRefSubCategory = NULL;
	gchar *pProductName = NULL;
	gchar *pProductPrice = NULL;
	gchar *pRefProduct = NULL;
	
	const gchar *pNewName;
	const gchar *pNewPrice;
	
	list = gtk_tree_selection_get_selected_rows(pPreferencesData->selectListProducts, NULL);
	
	if(list != NULL){
		depth = gtk_tree_path_get_depth(list->data);
		tabIndices = gtk_tree_path_get_indices(list->data);
		
		switch(depth){
			/* category selected */
			case 1:
				pPath = gtk_tree_path_new_from_indices(tabIndices[0], -1);
				gtk_tree_model_get_iter((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter, pPath);
				gtk_tree_model_get((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter,
  																				COLUMN_PRODUCTS_PREFERENCES, &pCategory,
  																				COLUMN_REFCATEGORY, &pRefCategory,
  																				-1);
				
				pDialog = gtk_dialog_new_with_buttons("Modifier la catégorie",
																							GTK_WINDOW(pPreferencesData->pGlda->pWindow),
																							GTK_DIALOG_MODAL,
																							GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
																							GTK_STOCK_OK, GTK_RESPONSE_OK,
																							NULL);
				pTable = gtk_table_new(1, 2, FALSE);
				pLabel = gtk_label_new("Nom : ");
				gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
				pEntry = gtk_entry_new();
				gtk_entry_set_text(GTK_ENTRY(pEntry), pCategory);
				gtk_table_attach_defaults(GTK_TABLE(pTable), pEntry, 1, 2, 0, 1);
				gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pDialog)->vbox), pTable, TRUE, FALSE, 0);
				gtk_widget_show_all(GTK_DIALOG(pDialog)->vbox);
				
				g_free(pCategory);
				pCategory = NULL;
				break;
			/* subcategory selected */
			case 2:
				pPath = gtk_tree_path_new_from_indices(tabIndices[0], tabIndices[1], -1);
				gtk_tree_model_get_iter((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter, pPath);
				gtk_tree_model_get((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter,
  																				COLUMN_PRODUCTS_PREFERENCES, &pSubCategory,
  																				COLUMN_REFCATEGORY, &pRefCategory,
  																				COLUMN_REFSUBCATEGORY, &pRefSubCategory,
  																				-1);
				
				pDialog = gtk_dialog_new_with_buttons("Modifier la sous-catégorie",
																								GTK_WINDOW(pPreferencesData->pGlda->pWindow),
																								GTK_DIALOG_MODAL,
																								GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
																								GTK_STOCK_OK, GTK_RESPONSE_OK,
																								NULL);
				pTable = gtk_table_new(1, 2, FALSE);
				pLabel = gtk_label_new("Nom : ");
				gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
				pEntry = gtk_entry_new();
				gtk_entry_set_text(GTK_ENTRY(pEntry), pSubCategory);
				gtk_table_attach_defaults(GTK_TABLE(pTable), pEntry, 1, 2, 0, 1);
				gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pDialog)->vbox), pTable, TRUE, FALSE, 0);
				gtk_widget_show_all(GTK_DIALOG(pDialog)->vbox);
				
				g_free(pSubCategory);
				pSubCategory = NULL;
				break;
			/* product selected */
			case 3:
				pPath = gtk_tree_path_new_from_indices(tabIndices[0], tabIndices[1], tabIndices[2], -1);
				gtk_tree_model_get_iter((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter, pPath);
				gtk_tree_model_get((GtkTreeModel*)pPreferencesData->pTreeListProducts, &Iter,
  																				COLUMN_PRODUCTS_PREFERENCES, &pProductName,
  																				COLUMN_PRICE_PREFERENCES, &pProductPrice,
  																				COLUMN_REFCATEGORY, &pRefCategory,
  																				COLUMN_REFSUBCATEGORY, &pRefSubCategory,
  																				COLUMN_REFPRODUCTS, &pRefProduct,
  																				-1);
				
				pDialog = gtk_dialog_new_with_buttons("Modifier l'article",
																								GTK_WINDOW(pPreferencesData->pGlda->pWindow),
																								GTK_DIALOG_MODAL,
																								GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
																								GTK_STOCK_OK, GTK_RESPONSE_OK,
																								NULL);
				pTable = gtk_table_new(2, 2, FALSE);
				pLabel = gtk_label_new("Nom : ");
				gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 0, 1);
				pEntryName = gtk_entry_new();
				gtk_entry_set_text(GTK_ENTRY(pEntryName), pProductName);
				gtk_table_attach_defaults(GTK_TABLE(pTable), pEntryName, 1, 2, 0, 1);
				pLabel = gtk_label_new("Prix :");
				gtk_table_attach_defaults(GTK_TABLE(pTable), pLabel, 0, 1, 1, 2);
				pEntryPrice = gtk_entry_new();
				gtk_entry_set_text(GTK_ENTRY(pEntryPrice), pProductPrice);
				gtk_table_attach_defaults(GTK_TABLE(pTable), pEntryPrice, 1, 2, 1, 2);
				gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pDialog)->vbox), pTable, TRUE, FALSE, 0);
				gtk_widget_show_all(GTK_DIALOG(pDialog)->vbox);
				
				g_free(pProductName);
				g_free(pProductPrice);
				pProductName = NULL;
				pProductPrice = NULL;
				break;
			default:
				break;
		}
		
		switch (gtk_dialog_run(GTK_DIALOG(pDialog))){
  		case GTK_RESPONSE_OK:
  			pPreferencesData->isNeedReloadList = TRUE;
  			switch(depth){
  				case 1:
  					pNewName = gtk_entry_get_text(GTK_ENTRY(pEntry));
  					modifyCategoryFromListProducts(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory, pNewName);
  					gtk_tree_store_set(pPreferencesData->pTreeListProducts, &Iter,
  															COLUMN_PRODUCTS_PREFERENCES, pNewName,
  															-1);
  					break;
  				case 2:
  					pNewName = gtk_entry_get_text(GTK_ENTRY(pEntry));
  					ModifySubCategoryFromListProducts(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory, pRefSubCategory, pNewName);
  					gtk_tree_store_set(pPreferencesData->pTreeListProducts, &Iter,
  															COLUMN_PRODUCTS_PREFERENCES, pNewName,
  															-1);
  					break;
  				case 3:
  					pNewName = gtk_entry_get_text(GTK_ENTRY(pEntryName));
  					pNewPrice = gtk_entry_get_text(GTK_ENTRY(pEntryPrice));
  					pProductPrice = g_strdup_printf("%.2f", strtod(pNewPrice, NULL));
  					modifyProductFromListProducts(pPreferencesData->pGlda->xmlDocListProducts, pRefCategory, pRefSubCategory, pRefProduct, pNewName, pProductPrice);
  					gtk_tree_store_set(pPreferencesData->pTreeListProducts, &Iter,
  															COLUMN_PRODUCTS_PREFERENCES, pNewName,
  															COLUMN_PRICE_PREFERENCES, pProductPrice,
  															-1);
  					g_free(pProductPrice);
  					break;
  			}
  			gtk_widget_destroy(pDialog);
  			break;
  		case GTK_RESPONSE_CANCEL:
  		default:
  			gtk_widget_destroy(pDialog);
  			break;
  	}
  	if(pRefCategory)
  		g_free(pRefCategory);
  	if(pRefSubCategory)
  		g_free(pRefSubCategory);
  	if(pRefProduct)
  		g_free(pRefProduct);
	}
}

/********************************** Callbacks **********************************/

/*** callback for notebook "General" ***/
void OnChangedDevise(GtkWidget *pEntry, Preferences *pPreferencesData){
	const gchar *pDevise;
	gchar *pText;
	gboolean isSave;
	
	pDevise = gtk_entry_get_text(GTK_ENTRY(pEntry));
	/* Change config's file */
	g_key_file_set_string(pPreferencesData->pGlda->pConfig, "general", "devise", pDevise);
	
	isSave = pPreferencesData->pGlda->isSave;
	
	/* Change on interface */
	pText = g_strdup_printf("Prix (en %s)", pDevise);
	gtk_tree_view_column_set_title(pPreferencesData->pGlda->pColumnPriceProducts, pText);
	gtk_tree_view_column_set_title(pPreferencesData->pGlda->pColumnPriceProductsAdded, pText);
	refreshTotalPriceAndQuantity(pPreferencesData->pGlda);
	pPreferencesData->pGlda->isSave = isSave;
}

void OnClickedShowColumnPriceProduct(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "product", "show_column_price", keyBool);
	
	/* Change on interface */
	gtk_tree_view_column_set_visible(pPreferencesData->pGlda->pColumnPriceProducts, keyBool);
	gtk_tree_view_column_set_visible(pPreferencesData->pGlda->pColumnPriceProductsAdded, keyBool);
}

void OnClickedShowColumnQuantityProduct(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "product", "show_column_quantity", keyBool);
	
	/* Change on interface */
	gtk_tree_view_column_set_visible(pPreferencesData->pGlda->pColumnQuantityProductsAdded, keyBool);
}

void OnClickedShowTotalQuantityProduct(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "product", "show_total_quantity", keyBool);
	
	/* Change on interface */
	pPreferencesData->isNeedReboot = TRUE;
}

void OnClickedShowTotalPriceProduct(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "product", "show_total_price", keyBool);
	
	/* Change on interface */
	pPreferencesData->isNeedReboot = TRUE;
}

/*** callback for notebook "print" ***/
void OnSelectFontPrint(GtkWidget *pButton, Preferences *pPreferencesData){
	createFontSelectionDialog(pPreferencesData);
}

void OnSelectColorPrint(GtkWidget *pButton, Preferences *pPreferencesData){
	createColorSelectionDialog(pPreferencesData);
}

void OnClickedShowTitlePrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_title", keyBool);
}

void OnClickedShowPagePrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_page", keyBool);
}

void OnValueChangedBorder(GtkWidget *pSpin, Preferences *pPreferencesData){
	gint keyInteger;
	
	keyInteger = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pSpin));
	
	/* Change config's file */
	g_key_file_set_integer(pPreferencesData->pGlda->pConfig, "print", "border", keyInteger);
}

void OnClickedShowColumnNumberPrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_number", keyBool);
}

void OnClickedShowColumnNamePrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_name", keyBool);
}

void OnClickedShowColumnPricePrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_price", keyBool);
}

void OnClickedShowColumnQuantityPrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_column_quantity", keyBool);
}

void OnClickedShowLineTotalPrint(GtkWidget *pCheckButton, Preferences *pPreferencesData){
	gboolean keyBool;
	
	keyBool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCheckButton));
	
	/* Change config's file */
	g_key_file_set_boolean(pPreferencesData->pGlda->pConfig, "print", "show_line_total", keyBool);
}

/*** callback for notebook "List of product" ***/
void OnSelectFileListProducts(GtkWidget *pButton, Preferences *pPreferencesData){
	GtkWidget *pFileSelection;
  gchar *pathFile;
  xmlDocPtr xmlDocListProducts;
  
  pathFile = g_key_file_get_string(pPreferencesData->pGlda->pConfig, "file", "file_list_product", NULL);
  pFileSelection = gtk_file_chooser_dialog_new("Selection le fichier lda...",
  									GTK_WINDOW(pPreferencesData->pGlda->pWindow),
  									GTK_FILE_CHOOSER_ACTION_OPEN,
  									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
  									GTK_STOCK_OPEN, GTK_RESPONSE_OK,
  									NULL);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(pFileSelection), pathFile);
	g_free(pathFile);
	pathFile = NULL;
	
	switch(gtk_dialog_run(GTK_DIALOG(pFileSelection))){
		case GTK_RESPONSE_OK:
			pathFile = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pFileSelection));
			xmlDocListProducts = initListProducts(pathFile);
			if(xmlDocListProducts == NULL){
				dialogError(pPreferencesData->pGlda, "Fichier lda incorrect");
			}else{
				gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryListProducts), pathFile);
				g_key_file_set_string(pPreferencesData->pGlda->pConfig, "file", "file_list_product", pathFile);
				pPreferencesData->isNeedReboot = TRUE;
				freeListProducts(xmlDocListProducts);
			}
			g_free(pathFile);
			gtk_widget_destroy(pFileSelection);
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			gtk_widget_destroy(pFileSelection);
			break;
	}
}

void OnClearCategory(GtkWidget *pButton, Preferences *pPreferencesData){
	gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryCategory), "");
}

void OnAddCategory(GtkWidget *pButton, Preferences *pPreferencesData){
	const gchar *pCategory;
	
	pCategory = gtk_entry_get_text(GTK_ENTRY(pPreferencesData->pEntryCategory));
	if(strcmp(pCategory, "") != 0){
		pPreferencesData->isNeedReloadList = TRUE;
		
		addNewCategoryToListProducts(pPreferencesData->pGlda->xmlDocListProducts, pCategory);
		
		/* update list on interface */
		gtk_combo_box_append_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInSubCategory), pCategory);
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInSubCategory), 0);
		gtk_combo_box_append_text(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), pCategory);
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), 0);
		
		gtk_tree_store_clear(pPreferencesData->pTreeListProducts);
		createListProducts(pPreferencesData);
		gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryCategory), "");
	}
}

void OnClearSubCategory(GtkWidget *pButton, Preferences *pPreferencesData){
	gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntrySubCategory), "");
  gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInSubCategory), 0);
}

void OnAddSubCategory(GtkWidget *pButton, Preferences *pPreferencesData){
	gint positionCategory;
	const gchar *pSubCategory;
	
	positionCategory = gtk_combo_box_get_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInSubCategory));
	pSubCategory = gtk_entry_get_text(GTK_ENTRY(pPreferencesData->pEntrySubCategory));
	if(positionCategory != -1 && strcmp(pSubCategory, "") != 0){
		pPreferencesData->isNeedReloadList = TRUE;
		
		positionCategory++;
		addNewSubCategoryToListProducts(pPreferencesData->pGlda->xmlDocListProducts, positionCategory, pSubCategory);
		
		/* update list on interface */
		gtk_tree_store_clear(pPreferencesData->pTreeListProducts);
		createListProducts(pPreferencesData);
		gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntrySubCategory), "");
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInSubCategory), 0);
		
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), -1);
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), 0);
	}
}

void OnClearProduct(GtkWidget *pButton, Preferences *pPreferencesData){
	gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryProductName), "");
  gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryProductPrice), "");
  gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), 0);
}

void OnAddProduct(GtkWidget *pButton, Preferences *pPreferencesData){
	gint positionCategory, positionSubCategory;
	const gchar *pProductName;
	const gchar *pProductPrice;
	gchar *pPrice = NULL;
	
	positionCategory = gtk_combo_box_get_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct));
	positionSubCategory = gtk_combo_box_get_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct));
	pProductName = gtk_entry_get_text(GTK_ENTRY(pPreferencesData->pEntryProductName));
	pProductPrice = gtk_entry_get_text(GTK_ENTRY(pPreferencesData->pEntryProductPrice));
	
	if(positionCategory != -1 && positionSubCategory != -1 && strcmp(pProductName, "") != 0 && strcmp(pProductPrice, "") != 0){
		pPreferencesData->isNeedReloadList = TRUE;
		
		pPrice = g_strdup_printf("%.2f", strtod(pProductPrice, NULL));
		positionCategory++;
		positionSubCategory++;
		addNewProductToListProducts(pPreferencesData->pGlda->xmlDocListProducts, positionCategory, positionSubCategory, pProductName, pPrice);
		g_free(pPrice);
		
		/* update list on interface */
		gtk_tree_store_clear(pPreferencesData->pTreeListProducts);
		createListProducts(pPreferencesData);
		
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct), 0);
		gtk_combo_box_set_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxSubCategoryInProduct), 0);
		gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryProductName), "");
		gtk_entry_set_text(GTK_ENTRY(pPreferencesData->pEntryProductPrice), "");
	}
}

void OnChangedComboBoxCategory(GtkWidget *pComboBox, Preferences *pPreferencesData){
	gint position;
	
	position = gtk_combo_box_get_active(GTK_COMBO_BOX(pPreferencesData->pComboBoxCategoryInProduct));
	position++;
	
	changeComboBoxSubCategory(pPreferencesData, position);
}

void OnRemoveFromList(GtkWidget *pButton, Preferences *pPreferencesData){
	removeFromList(pPreferencesData);
}

void OnEditList(GtkWidget *pButton, Preferences *pPreferencesData){
	editList(pPreferencesData);		
}
